/* matrix.h - matrix stuff and graphic types
 * Patrick Angeles
 *
 */

#ifndef MATRIX_H
#define MATRIX_H

#define X 0
#define Y 1
#define Z 2
#define H 3

typedef struct
{
   double          x, y, z;
}               point;

typedef double  vector[4];
typedef double  matrix[4][4];


#define VX(c)   (c)[ 0]
#define VY(c)   (c)[ 1]
#define VZ(c)   (c)[ 2]
#define VW(c)   (c)[ 3]

#define PX(c)   (c).x
#define PY(c)   (c).y
#define PZ(c)   (c).z

#define SQR(x)  ((x)*(x))
#define CUBE(x) ((x)*(x)*(x))

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif


#define USE_MACROS

extern void     mult_matrix();
extern void     zero_matrix();
extern void     id_matrix();
extern void     copy_matrix();
extern void     vector_transform();
extern void     translate();
extern void     scale();
extern void     rotate_x();
extern void     rotate_y();
extern void     rotate_z();
extern void     print_matrix();
extern void     unit_vector();
extern void     unit_line_vector();
extern void     normalized_cross_product();

#ifndef USE_MACROS
/************ function prototypes ************/
extern void     vector_copy();
extern void     point_copy();
extern void     double2point();
extern void     point2double();
extern void     double2vector();
extern void     vector2double();
extern void     point2vector();
extern void     vector2point();
extern void     homogenize();
extern void     vector_init();
extern void     vector_zero();
extern void     dot_product();

#else
/************ macro types *******************/
#define vector_copy(t,s) { \
        (t)[0]=(s)[0];(t)[1]=(s)[1];(t)[2]=(s)[2];(t)[3]=(s)[3];};
#define point_copy(t,s) { \
        (t)->x=(s)->x;(t)->y=(s)->y;(t)->z=(s)->z;};
#define double2point(d,v1,v2,v3) { \
        (d)->x=v1;(d)->y=v2;(d)->z=v3;};
#define point2double( s,v1,v2,v3) { \
        *(v1)=(s)->x;*(v2)=(s)->y;*(v3)=(s)->z;};
#define double2vector(d,v1,v2,v3) { \
        (d)[0]=(v1);(d)[1]=(v2);(d)[2]=(v3);(d)[3]=1.0;};
#define vector2double(s,v1,v2,v3) { \
        *(v1)=(s)[ 0];*(v2)=(s)[ 1];*(v3)=(s)[ 2];};
#define point2vector(d,s) { \
        (d)[0]=(s)->x;(d)[1]=(s)->y;(d)[2]=(s)->z;(d)[3]=1.0;};
#define vector2point(d,s) { \
        (d)->x=(s)[0];(d)->y=(s)[1];(d)->z=(s)[2];};
#define homogenize(d) { \
        (d)[0]/=(d)[ 3];(d)[1]/=(d)[ 3];(d)[2]/=(d)[ 3];(d)[3]=1.0;};
#define vector_init(d) { \
        (d)[0]=0.0;(d)[1]=0.0;(d)[2]=0.0;(d)[3]=1.0;};
#define vector_zero(d) { \
        (d)[0]= 0.0;(d)[1]= 0.0;(d)[2]= 0.0;(d)[3]= 0.0;};
#define line_vector(d,h,t) { \
        (d)[0]=(h)->x-(t)->x;(d)[1]=(h)->y-(t)->y; \
        (d)[2]=(h)->z-(t)->z;(d)[3]=1.0;};
#define dot_product(d,v1,v2) { \
        *(d)= (double)((v1)[0]*(v2)[0]+(v1)[1]*(v2)[1]+(v1)[2]*(v2)[2]);};
#endif

/****** not yet functions...   ***************/

#define vector_scale(t,s,S) { \
        (t)[0]=(s)[0]*(S);(t)[1]=(s)[1]*(S); \
        (t)[2]=(s)[2]*(S);(t)[3]=1.0;};
#define vector_add(t,s1,s2) { \
        (t)[0]=(s1)[0]+(s2)[0];(t)[1]=(s1)[1]+(s2)[1]; \
        (t)[2]=(s1)[2]+(s2)[2];(t)[3]=1.0;};
#define vector_sub(t,s1,s2) { \
        (t)[0]=(s1)[0]-(s2)[0];(t)[1]=(s1)[1]-(s2)[1]; \
        (t)[2]=(s1)[2]-(s2)[2];(t)[3]=1.0;};
#define point_move(p,v) { \
        (p)->x+=(v)[0];(p)->y+=(v)[1];(p)->z+=(v)[2];};
#endif

void homogenize_vector_transform(vector v, matrix Matrix);
void vector_transform1(vector Vector, matrix Matrix);

